/**
 * @internal
 */
export interface MessageParts {
    headers: DataView;
    body: Uint8Array;
}
/**
 * @internal
 */
export declare function splitMessage({ byteLength, byteOffset, buffer }: ArrayBufferView): MessageParts;
